<?php
namespace Teknisa\SUP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\SUP\Modules\Cadastros\Services\ConfigAlcadasService;
use Teknisa\SUP\Modules\Cadastros\Services\FilialAlcadasService;
use Teknisa\SUP\Modules\Cadastros\Services\AprovadorService;
use Teknisa\SUP\Modules\Cadastros\Services\ConfigAlcadasSheetImporter;
use Zeedhi\Framework\DataSource\DataSet;

class ConfigAlcadasController
{
    const CHAVES_PARA_IGNORAR = array(
        '$error',
        '__createdLocal',
        '__is_new',
        '__index',
        '__isSelected',
        'changed',
    );
    
    public function __construct(
        ConfigAlcadasService $service,
        FilialAlcadasService $serviceFilialAlcada,
        AprovadorService $serviceAprovador,
        ConfigAlcadasSheetImporter $configAlcadasSheetImporter,
        $smtpUtil
    ) {
        $this->service             = $service;
        $this->serviceFilialAlcada = $serviceFilialAlcada;
        $this->serviceAprovador    = $serviceAprovador;
        $this->sheetImporter       = $configAlcadasSheetImporter;
        $this->smtpUtil            = $smtpUtil;
    }
    
    public function saveNewConfigAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $row = self::formataValorMoedaParaFloat($row);
        $row = self::removerChavesIgnoradas($row);
        
        try {
            $this->service->saveConfigAlcada($row);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveEditConfigAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        $row                      = $request->getRow();
        $updatedConfigAlcada      = $row['NEW'];
        $oldConfigAlcada          = $row['OLD'];
        
        $updatedConfigAlcada      = self::formataValorMoedaParaFloat($updatedConfigAlcada);
        $oldConfigAlcada          = self::formataValorMoedaParaFloat($oldConfigAlcada);
        
        try {
            $propriedadesAlteradas = self::getPropriedadesAlteradas($updatedConfigAlcada, $oldConfigAlcada);
                
            $qtdPropriedadesAlteradas = count($propriedadesAlteradas);
            
            if ($qtdPropriedadesAlteradas == 0) {
                throw new \Exception('THERE_ARE_NO_CHANGES');
            }
            
            $numeroConfigAlcada = $updatedConfigAlcada['NRCONFIGALCADA'];
            
            if (self::possuiAprovacoesPendentes($numeroConfigAlcada)) {
                if (!array_key_exists('NMALCADA', $propriedadesAlteradas) || $qtdPropriedadesAlteradas > 1) {
                    throw new \Exception('OPERATION_BLOCKED_SAVE_COMPETENCE_PENDING_APPROVALS');
                }
            }
            
            $isTipoAlcadaAlterado     = array_key_exists('IDTPALCADA', $propriedadesAlteradas);
            $isExigeAprovacaoAlterado = array_key_exists('IDEXIGETAPROVE', $propriedadesAlteradas);
            
            if ($isTipoAlcadaAlterado || $isExigeAprovacaoAlterado) {
                $this->serviceAprovador->resetarValorOrdemAprovacaoTodosAprovadoresDaAlcada($updatedConfigAlcada);
                $response->addMessage(new Message('VALUES_HAS_BEEN_REDEFINED'));
            }
            
            $this->service->updateConfigAlcada($updatedConfigAlcada);
            
            /**
             * A função abaixo é a forma padrão de salvar o log, porém, atualmente, os logs são salvos em arquivos
             * em vez de banco. Porém, essa funcionalidade depende da criação da tabela de log e, portanto,
             * encontra-se comentada até que seja filizada sua implementação no ConfigAlcadasService.
             * 
             * A função que a substitui se chama saveLogAlteracaoArquivo, encontrada no service.
             */
            // $this->salvarAlteracoesEmLog($numeroConfigAlcada, $propriedadesAlteradas);
            
            // $this->service->saveLogAlteracaoArquivo($numeroConfigAlcada, $propriedadesAlteradas);
            
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteConfigAlcada(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        try {
            if (self::possuiAprovacoesPendentes($row['NRCONFIGALCADA'])) {
                throw new \Exception('OPERATION_BLOCKED_DEL_COMPETENCE_PENDING_APPROVALS');
            }
            
            $this->serviceAprovador->deleteTodosAprovadoresRelacionadosAlcada($row);
            $this->serviceAprovador->deleteTodosAprovadoresAlcada($row);
            $this->serviceFilialAlcada->deleteTodasFiliaisAlcada($row);
            $this->service->deleteConfigAlcada($row);   
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    /**
     * @salvarAlteracoesEmLog
     * Atualmente, os logs são salvos em arquivos em vez de banco. O padrão
     * é ser salvo em banco, porém, depende da criação da tabela de log.
     * 
     * A função saveLogAlteracao do service já está criada de forma incompleta.
     * Sua funcionalidade será implementada assim que houver os recursos necessários.
     */
    // public function salvarAlteracoesEmLog($numeroConfigAlcada, $propriedadesAlteradas)
    // {
    //     foreach ($propriedadesAlteradas as $nomePropriedade => $fraseDeAlteracao) {
    //         $this->service->saveLogAlteracao($numeroConfigAlcada, $nomePropriedade, $fraseDeAlteracao);
    //     }
    // }
    
    public function formataValorMoedaParaFloat($row)
    {
        if (array_key_exists('IDIMPGESTFIL', $row) && $row['IDIMPGESTFIL'] == 'S') {
            $fieldsName = array(
                'VRDIRETORIA',
                'VRGERREGIONAL',
                'VRSUPERVISOR',
                'VRPLANEJAMENTO',
            );
            $arrayReplace = array(
                'R$',
                '.',
            );
            
            foreach ($fieldsName as $name) {
                $row[$name] = str_replace(',', '.', str_replace($arrayReplace, '', $row[$name]));
            }
        }
        
        return $row;
    }
    
    public function possuiAprovacoesPendentes($NRCONFIGALCADA)
    {
        return $this->service->findAlcada($NRCONFIGALCADA)['APROVACAO'] > 0;
    }
    
    public function removerChavesIgnoradas($row)
    {
        if (method_exists($row, 'getArrayCopy')) {
            $row = array_filter($row->getArrayCopy());
        }
        
        $row = array_diff_key($row, array_fill_keys(self::CHAVES_PARA_IGNORAR, ''));
        
        return $row;
    }
    
    public function getPropriedadesAlteradas($updatedConfigAlcada, $oldConfigAlcada)
    {
        $propriedadesAlteradas = array();
        
        foreach ($updatedConfigAlcada as $key => $newValue) {
            if (array_key_exists($key, $oldConfigAlcada)) {
                $oldValue = $oldConfigAlcada[$key];
                
                if ($newValue != $oldValue) {
                    $propriedadesAlteradas[$key] = "alterada de '" . $oldValue . "' para '" . $newValue . "'";
                }
            }
        }
        
        return $propriedadesAlteradas;
    }
    
    public function enviaEmailQrcode(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $listaEmails = self::retornaListaEmailsQrcode($row['NRCONFIGALCADA'], $row['CDFILIAL'], $row['CDOPERADOR'], $row['CDOPERRELACAP']);
        
        $qrcodeIMG = $row['qrCode']['data'];
        
        $corpoEmail = "<!DOCTYPE html><html><head><meta charset='UTF-8'><style>body{margin:0px;display:block;padding:0px;margin:0px;font-family:Arial,sans-serif}.containerGeral{background-color:#FFF}.linha{width:590px;display:table}.cabecalho{background-color:#FFF;padding:5px 0px 5px 0px}.tipoEmail{background-color:#e6f2ff}.conteudo{display:table-row;width:550px;background-color:#FFF}.rodape{background-color:#2D68C4}.logomarca{padding:15px;vertical-align:middle}#assunto{padding:25px 50px 25px 50px;vertical-align:middle;text-align:left;color:#00305A;font-size:23px}#borda{vertical-align:middle}#quadroConteudo{width:550px;line-height:150%;font-size:14px;padding:20px;color:#726969}#direitosReservados{padding:20px 0px 20px 0px;color:#FFF;font-size:15px;text-align:center}</style></head><body><center class='containerGeral'><table cellpadding='0' cellspacing='0'><tbody><tr class='linha cabecalho'><td class='logomarca'> <img id='logo' width='220' src='http://midiatest.teknisa.com/files/tkOrg058b72c1f04e58/Sup/MailImg/5b16e099e4679.png'></td></tr><tr class='linha tipoEmail'><td id='assunto'><p>". $row['SUBJECT'] ."</p></td></tr><div id='quadroConteudo'><tr class='linha conteudo'><td id='borda'><center><table cellpadding='0' cellspacing='0' style='background-color: #f2f2f2; width: 590px;'><tbody><tr><td><div style='margin: 20px; word-wrap: break-word;'> ". $row['MESSAGE'] ."</div><center> <img src='". $qrcodeIMG ."' alt='qrcode' style='margin: 20px'></center></td></tr></tbody></table></td></tr></div><tr class='linha rodape'><td id='direitosReservados'><p>". $row['ALL_RIGHTS'] . "</p></td></tr></tbody></table></body></html>";
        
        $this->smtpUtil->setSubject($row['SUBJECT']);
        $this->smtpUtil->sendMail($corpoEmail, $listaEmails);
        
        $response->addDataSet(new DataSet("listaEmails", $listaEmails));
    }
    
    function retornaListaEmailsQrcode($nrconfigalcada, $cdfilial, $cdoperador, $cdoperrelacap)
    {
        $operadores = $this->service->retornaOperadoresAlcada($nrconfigalcada, $cdfilial, $cdoperador, $cdoperrelacap);
        
        $emails = array_map(function($operador) {
            return $operador['DSEMAILOPER'];
        }, $operadores);
        
        $emails = array_unique($emails);
        
        $emails = array_filter($emails, function($email) {
            return $email;
        });
        
        return $emails;
    }
    
    function getIdSisDos(DTO\Request\Row $request, DTO\Response $response)
    {
        $idSisDos = $this->service->getIdSisDos();
        
        $response->addDataSet(new DataSet("callback", $idSisDos));
    }
    
    public function importSpreadsheet(DTO\Request $request, DTO\Response $response) {
		try {
		    $this->sheetImporter->beginTransaction();
			$row = $request->getRow();
		    
			$sheetPath = $this->sheetImporter->createTemporaryFile($row["SPREADSHEET"]);
			$worksheet = $this->sheetImporter->loadSheet($sheetPath);
			$sheet = $this->sheetImporter->parseSheet($worksheet);
			
			$this->sheetImporter->validateSheetColumns($sheet);
			$this->sheetImporter->validateSheetRows($sheet, $response);
			
			$sequential = $this->sheetImporter->validateAndInsertData($sheet, $response);
			
			$this->sheetImporter->startImportProcedure($sequential);
			$this->sheetImporter->commitTransaction();
		    
		    $errors = $this->sheetImporter->checkForErrors($sequential);
		    
		    
		    if (isset($errors[0])) {
		        $response->addDataSet(new DataSet("errors", $errors));
		    }
		} catch (\Exception $e) {
		    $this->sheetImporter->rollbackTransaction();
			$response->setError(new Error($e->getMessage(), 500));
		} finally {
			if (isset($sheetPath)) {
                $this->sheetImporter->removeTemporaryFile($sheetPath);
            }
		}
    }
}
